'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "lib"
directory.
'''
import data
import array
import random
import math
import pygame

DELAY = 15
XDIM = 320
YDIM = 240
MAXCODA = 10000
mucusCOL = 100
wallCOL = 200
BASECOL = 65

TABSIZE = 2048
TABAND = 2047
T_PI = 1024
T_PI2 = 512


class Global(object):
    
    def __init__(self):
        return

glob = Global()


 
class Player(object):
    
    def __init__(self,position,color):
        self.queue = []
        self.angle = 0
        self.dim = 50
        self.color = color
        self.oldx = int(position[0])
        self.oldy = int(position[1])
        self.head(position)
        
    def head(self,(position)):
        x = int(position[0])
        y = int(position[1])
        self.place(x,y)
        if glob.isdead: return
        self.place(x+1,y)
        if glob.isdead: return
        self.place(x,y+1)
        if glob.isdead: return
        self.place(x-1,y)
        if glob.isdead: return
        self.place(x,y-1)
        if glob.isdead: return
        self.queue.append(position)
        
    def tail(self):
        return self.queue.pop(0)
        
    def place(self,x,y):
    
        if x<0 or x>=XDIM or y<0 or y>=YDIM:
            glob.isdead=1
            putquote("WORM %d LOST IN THE VOID" % (self.color-BASECOL))
            return
            
        c = glob.dbuf[x][y]
    
        if c:
    
            if c == self.color:
            
                dx = abs(x-self.oldx)
                dy = abs(y-self.oldy)
            
                if not (dx<2 and dy<2):
                    glob.isdead=1
                    putquote("WORM %d SPLATTED INTO ITSELF" % (self.color-BASECOL+1))
                    return
                 
            elif c == mucusCOL:
            
                self.dim+=glob.mucusr<<3
                putmucus(glob.mucusx,glob.mucusy,glob.mucusr,0)
                putstatus()
                placemucus()
                
            elif c == wallCOL:
            
                glob.isdead=1
                putquote("WORM %d SPLATTED INTO THE DIRT" % (self.color-BASECOL+1))
                return
               
            else:
            
                glob.isdead=1
                putquote("WORM %d SPLATTED INTO WORM %d" % (self.color-BASECOL+1,c-BASECOL+1))
                return
            
        
        setdbuf(x,y,self.color)

    
    def step(self):
    
        p = self.queue[len(self.queue)-1]
         
        self.oldx = int(p[0])
        self.oldy = int(p[1])
        
        nx = p[0]+sint(self.angle)
        ny = p[1]+cost(self.angle)
                
        self.head((nx,ny))
        
        if len(self.queue)>self.dim:
            
            o = self.tail()
            x = int(o[0])
            y = int(o[1])
               
            resetdbuf(x,y)
            resetdbuf(x+1,y)
            resetdbuf(x,y+1)
            resetdbuf(x-1,y)
            resetdbuf(x,y-1)
         
        

def sint(i):
    return glob.stab[i&TABAND]

def cost(i):
    return glob.stab[(i+T_PI2)&TABAND]


def setplasma():
   
  for y in range (0,YDIM):
      for x in range (0,XDIM):
          glob.plasma[x][y]=int((sint(x<<4)*cost(y<<4))*32.0)+32
 

def putpixel (x,y,c):
    glob.screen.set_at((x,y),c)
 

def gputchar (xx,yy,c,col,bak):
    
    mask = 0x80
    f = c<<3 
    
    for y in range (0,8):
        
        for x in range (0,8):
            
            if mask==0:
                mask = 0x80
                f+=1
                
            if glob.fontable[f]&mask:
                if bak>=0:
                    putpixel(xx+x,yy+y,bak)
            else:
                putpixel(xx+x,yy+y,col)
                
            mask>>=1
            

def gputstring (x,y,s,c):
    
    for car in s:
        gputchar(x,y,ord(car),c,0)
        x+=8


def putquote(s,y=0,col=mucusCOL):
    gputstring((XDIM>>1)-(len(s)<<2),y,s,col)
      
    
def putstatus():
    
    for i in range (0,glob.nplay):
        
        s = "PL%01d:%05d" % (i+1,glob.players[i].dim)
        gputstring(i*XDIM/glob.nplay,YDIM-8,s,BASECOL+i)
     
 
def resetdbuf(x,y):
     glob.dbuf[x][y]=0
     glob.screen.set_at((x,y),glob.plasma[x][y])

def setdbuf(x,y,col):
    glob.screen.set_at((x,y),col)
    glob.dbuf[x][y]=col 
 
def circput(x,y,xc,yc,col):
    
    if col:
        for i in range (xc-x,x+xc+1):
            setdbuf(i,y+yc,col)
            setdbuf(i,yc-y,col)
    
        for i in range (xc-y,y+xc+1):
            setdbuf(i,yc-x,col)
            setdbuf(i,x+yc,col)
        
    else:                         
        for i in range (xc-x,x+xc+1):
            resetdbuf(i,y+yc)
            resetdbuf(i,yc-y)
    
        for i in range (xc-y,y+xc+1):
            resetdbuf(i,yc-x)
            resetdbuf(i,x+yc)
   
     
def putmucus(xc,yc,ray,col):
    y=ray
    delta=3-(ray<<1)
    x=0
    
    while x<y:
        circput(x,y,xc,yc,col)
        if delta<0:
             delta+=4*x+6            
        else:  
             delta+=4*(x-y)+10
             y-=1
                 
        x+=1
   
    x=y
    if y:
        circput(x,y,xc,yc,col)
    
     
        
def placemucus():

    done=False
    
    while not done:
        
        r=random.randint(3,10)
        xp=random.randint(r,XDIM-r)
        yp=random.randint(r,YDIM-r)
        
        done = True
        
        for y in range (yp-r,yp+r):
            for x in range (xp-r,xp+r):
                if glob.dbuf[x][y]:
                    done = False
 
 
    glob.mucusx=xp
    glob.mucusy=yp
    glob.mucusr=r        
                    
    putmucus(xp,yp,r,mucusCOL)

   
    
def resetgame():
    
    for x in range (0,XDIM):
        for y in range (0,YDIM):
            glob.dbuf[x][y]=0
            
    pygame.surfarray.blit_array(glob.screen, glob.plasma)
             
    for x in range (0,XDIM):
        setdbuf(x,8,wallCOL)
        setdbuf(x,YDIM-9,wallCOL)
        
    for y in range (8,YDIM-9):
        setdbuf(0,y,wallCOL)
        setdbuf(XDIM-1,y,wallCOL)
 
    putquote("MUCUSNACK");

    glob.isdead = 0
 
    d=XDIM/(glob.nplay+1);
    glob.players = []
    x = d
    y = YDIM>>1 
    
    for i in range (0,glob.nplay):
        glob.players.append(Player((float(x),float(y)),BASECOL+i))
        x+=d
        
    placemucus()
    putstatus()
    

def main():

    glob.fontable = array.array('B')
    glob.fontable.read(data.load("font8x8"),1024)
    
    keybindings = ( (pygame.K_z,pygame.K_x) , (pygame.K_o,pygame.K_p) , (pygame.K_LEFT,pygame.K_RIGHT) )

    glob.stab = []
    for i in range (0,TABSIZE):
        glob.stab.append(math.sin(math.pi*i/T_PI))
        
    pygame.init()
    glob.screen = pygame.display.set_mode((XDIM,YDIM), pygame.HWSURFACE,8) #pygame.DOUBLEBUF,8)
    
    
    for i in range (0,65):   
        glob.screen.set_palette_at(i,[i<<1,i<<1,i<<1])
    
    glob.screen.set_palette_at(BASECOL,[255,255,0])
    glob.screen.set_palette_at(BASECOL+1,[255,0,255])
    glob.screen.set_palette_at(BASECOL+2,[0,255,255])
    glob.screen.set_palette_at(mucusCOL,[0,255,0])
    glob.screen.set_palette_at(wallCOL,[255,128,0])
    
    pygame.display.set_palette(glob.screen.get_palette())
    
    glob.plasma = pygame.surfarray.array2d(glob.screen)
    glob.dbuf = pygame.surfarray.array2d(glob.screen)
    setplasma()
   
    ticks = pygame.time.get_ticks()+DELAY
    random.seed()
    glob.isdead=1;
    
    putquote(" MUCUSNACK ",30)
    putquote(" PRESS 1,2 OR 3 TO START THE GAME ",50,wallCOL)

    putquote(" WORM1:  Z=LEFT       X=RIGHT       ",90,BASECOL)
    putquote(" WORM2:  O=LEFT       P=RIGHT       ",110,BASECOL+1)
    putquote(" WORM3:  CURSOR-LEFT  CURSOR-RIGHT  ",130,BASECOL+2)
    
    putquote(" EAT UP THE MUCUS, AVOID OTHER WORMS ",170,63)
    putquote(" DON'T BE GREEDY! ",190,63)
    putquote(" HIT ESC TO QUIT GAME ",210,63)
    
      
    while True:
       pygame.display.flip()
             
       event = pygame.event.poll()
        
       if event.type == pygame.QUIT:
           break
        
       elif event.type == pygame.KEYUP:

           if event.key == pygame.K_F8:
                pygame.image.save(glob.screen,'shot.png')

           if event.key == pygame.K_ESCAPE:
                break
               
           if event.key == pygame.K_1:
               glob.nplay = 1
               resetgame()
           if event.key == pygame.K_2:
               glob.nplay = 2
               resetgame()
           if event.key == pygame.K_3:
               glob.nplay = 3
               resetgame()
       
       if not glob.isdead:
   
            glob.keystatus = pygame.key.get_pressed()
        
            for i in range (0,glob.nplay):
       
                if glob.isdead: break
           
                play = glob.players[i]
           
                if glob.keystatus[keybindings[i][0]]:
                    play.angle=(play.angle+32)&TABAND;
                if glob.keystatus[keybindings[i][1]]:
                    play.angle=(play.angle-32)&TABAND;
                
                play.step()
               
 
       t = pygame.time.get_ticks()   
       if t<ticks:
           pygame.time.wait(ticks-t)    
       ticks += DELAY

    
    

    
